import type { Graph } from "../graph/graph.js";
import type { Node } from "../graph/node.js";
import type { Edge } from "../graph/edge.js";
export type GraphLayoutState = 'INIT' | 'START' | 'CALCULATING' | 'DONE' | 'ERROR';
export type GraphLayoutOptions = {};
/** All the layout classes are extended from this base layout class. */
export declare class GraphLayout<OptionsT extends GraphLayoutOptions = GraphLayoutOptions> extends EventTarget {
    /** Name of the layout. */
    protected readonly _name: string;
    /** Extra configuration options of the layout. */
    protected _options: OptionsT;
    version: number;
    state: GraphLayoutState;
    /**
     * Constructor of GraphLayout
     * @param options extra configuration options of the layout
     */
    constructor(options: OptionsT);
    /**
     * Check the equality of two layouts
     * @param layout - The layout to be compared.
     * @return - True if the layout is the same as itself.
     */
    equals(layout: GraphLayout): boolean;
    /** virtual functions: will be implemented in the child class */
    /** first time to pass the graph data into this layout */
    initializeGraph(graph: Graph): void;
    /** update the existing graph */
    updateGraph(graph: Graph): void;
    /** start the layout calculation */
    start(): void;
    /** update the layout calculation */
    update(): void;
    /** resume the layout calculation */
    resume(): void;
    /** stop the layout calculation */
    stop(): void;
    /** access the position of the node in the layout */
    getNodePosition(node: Node): [number, number];
    /** access the layout information of the edge */
    getEdgePosition(edge: Edge): {
        type: string;
        sourcePosition: number[];
        targetPosition: number[];
        controlPoints: any[];
    };
    /**
     * Pin the node to a designated position, and the node won't move anymore
     * @param  node Node to be locked
     * @param  x    x coordinate
     * @param  y    y coordinate
     */
    lockNodePosition(node: Node, x: number, y: number): void;
    /**
     * Unlock the node, the node will be able to move freely.
     * @param  {Object} node Node to be unlocked
     */
    unlockNodePosition(node: Node): void;
    protected _updateState(state: any): void;
    /** @fires GraphLayout#onLayoutStart */
    protected _onLayoutStart: () => void;
    /** @fires GraphLayout#onLayoutChange */
    protected _onLayoutChange: () => void;
    /** @fires GraphLayout#onLayoutDone */
    protected _onLayoutDone: () => void;
    /** @fires GraphLayout#onLayoutError */
    protected _onLayoutError: () => void;
}
//# sourceMappingURL=graph-layout.d.ts.map